import { Component, Inject, Injector, NgModule, ElementRef,
    SkipSelf, Optional, forwardRef, ChangeDetectorRef,
    EventEmitter } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { FormsModule, NG_VALUE_ACCESSOR } from '@angular/forms';
import { WjValueAccessorFactory } from 'wijmo/wijmo.angular2.directiveBase';
import { WjGridModule, WjFlexGrid, wjFlexGridMeta, 
    WjFlexGridColumn, wjFlexGridColumnMeta }  from 'wijmo/wijmo.angular2.grid';


@Component({
    selector: 'gc-grid',
    template: wjFlexGridMeta.template,
    inputs: [...wjFlexGridMeta.inputs, 'myProperty'],
    outputs: [...wjFlexGridMeta.outputs, 'myPropertyChange'],
    providers: [
        { provide: 'WjComponent', useExisting: forwardRef(() => GcGrid) },
        ...wjFlexGridMeta.providers
    ]
})
export class GcGrid extends WjFlexGrid {
  private _myProperty: string;
  myPropertyChange: EventEmitter<any>;

  // constructor(.....) {}
  //
  // We don't declare a constructor at all, in order to avoid necessity to maintain 
  // constructor parameters and keep them in synch with the base WjFlexGrid's constructor
  // parameters. Instead, we override the "created" method, which is called in the last
  // line of any Wijmo component's constructor, and perform necessary initializations here.
  created() {
      this._myProperty = '';
      this.myPropertyChange = new EventEmitter(false);
  }

  get myProperty(): string {
      return this._myProperty;
  }
  set myProperty(value: string) {
      if (this._myProperty !== value) {
          this._myProperty = value;
          this.myPropertyChange.emit(value);
      }
  }
}



@Component({
    selector: 'gc-column',
    template: wjFlexGridColumnMeta.template,
    inputs: wjFlexGridColumnMeta.inputs,
    outputs: wjFlexGridColumnMeta.outputs,
    providers: [
        { provide: 'WjComponent', useExisting: forwardRef(() => GcColumn) },
        ...wjFlexGridColumnMeta.providers
    ]
})
export class GcColumn extends WjFlexGridColumn {
}


@NgModule({
    imports: [WjGridModule, BrowserModule, FormsModule],
    declarations: [GcGrid, GcColumn],
    exports: [GcGrid, GcColumn]
})
export class GcGridModule {
}

